--
-- AnimatedObjectDoorFix
--
-- @author: Blacksheep(RC-Devil)
-- @version: 1.0.0.1
-- @date: 02 Nov 2017
-- Interface: 1.5.1.0 b1583
-- Notes: FS2017 Script for Fix Animated Objects to open Doors from Vehicle
-- Web: https://www.farming-mods.de
-- Support: https://www.mods-community.de
-- Copyright (C) 2017 Blacksheep(RC-Devil)

if not Utils.AnimatedObjectDoorFix4711 then
	Utils.AnimatedObjectDoorFix4711 = true;
	print('\\__ AnimatedObjectDoorFix V 1.0.0.1 by Blacksheep(RC-Devil) from 02 Nov 2017 is loading! Support over Forum: httpd://www.mods-community.de');
	
	local oldFunc = AnimatedObject.new
AnimatedObject.new = function(self, isServer, isClient, customMt)
    local mt = customMt
    if mt == nil then
        mt = AnimatedObject_mt
    end

    local self = Object:new(isServer, isClient, mt)
    self.nodeId = 0
    self.isMoving = false
    self.sendIsMoving = false
    self.wasPressed = false
    self.baseDirectory = nil
    self.customEnvironment = g_currentMission.loadingMapModName;
	self.playerInRange = false;
	
    return self
end

AnimatedObject.load = function(self, nodeId)
    self.nodeId = nodeId

    local xmlFilename = getUserAttribute(nodeId, "xmlFilename")

    if xmlFilename == nil then
        print("Error: Missing 'xmlFilename' user attribute for AnimatedObject node '"..getName(nodeId).."'!")
        return false
    end

    local baseDir = g_currentMission.loadingMapBaseDirectory
    if baseDir == "" then
        baseDir = Utils.getNoNil(self.baseDirectory, baseDir)
    end

    local i18n = g_i18n;
    if self.customEnvironment ~= nil then
        i18n = _G[self.customEnvironment].g_i18n;
    end

    self.xmlFilename = Utils.getFilename(xmlFilename, baseDir)
    local success = true
    local xmlFile = loadXMLFile("AnimatedObject", self.xmlFilename)
    if xmlFile ~= 0 then
        local index = getUserAttribute(nodeId, "index")
        if index ~= nil then
            local key = nil
            local i = 0
            while true do
                local objectKey = string.format("animatedObjects.animatedObject(%d)", i)
                if not hasXMLProperty(xmlFile, objectKey) then
                    break
                end

                local configIndex = getXMLString(xmlFile, objectKey.."#index")
                if configIndex == index then
                    key = objectKey
                    break
                end
                i = i + 1
            end

            if key ~= nil then
                self.saveId = getXMLString(xmlFile, key.."#saveId");
                if self.saveId == nil then
                    self.saveId = "AnimatedObject_"..getName(nodeId)
                end

                local animKey = key .. ".animation"

                self.animation = {}
                self.animation.parts = {}
                self.animation.duration = Utils.getNoNil(getXMLFloat(xmlFile, animKey.."#duration"), 3) * 1000
                if self.animation.duration == 0 then
                    self.animation.duration = 1000
                end
                self.animation.time = 0
                self.animation.direction = 0

                local i = 0
                while true do
                    local partKey = string.format("%s.part(%d)", animKey, i)
                    if not hasXMLProperty(xmlFile, partKey) then
                        break
                    end

                    local node = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, partKey.."#node"))
                    if node ~= nil then
                        local part = {}
                        part.node = node
                        part.animCurve = AnimCurve:new(linearInterpolatorN)
                        local hasFrames = false
                        local j = 0
                        while true do
                            local frameKey = string.format("%s.keyFrame(%d)", partKey, j)
                            if not hasXMLProperty(xmlFile, frameKey) then
                                break
                            end

                            local keyTime = getXMLFloat(xmlFile, frameKey.."#time")
                            local values = {self:loadFrameValues(xmlFile, frameKey, node)}
                            part.animCurve:addKeyframe({ v=values, time = keyTime})
                            hasFrames = true

                            j = j + 1
                        end

                        if hasFrames then
                            table.insert(self.animation.parts, part)
                        end
                    end
                    i = i + 1
                end

                local initialTime = Utils.getNoNil(getXMLFloat(xmlFile, animKey.."#initialTime"), 0)*1000
                self:setAnimTime(initialTime/self.animation.duration)

                local startTime = getXMLFloat(xmlFile, key..".openingHours#startTime")
                local endTime = getXMLFloat(xmlFile, key..".openingHours#endTime")
                if startTime ~= nil and endTime ~= nil then
                    local disableIfClosed = Utils.getNoNil(getXMLBool(xmlFile, key..".openingHours#disableIfClosed"), false)
                    local closedText = getXMLString(xmlFile, key..".openingHours#closedText")
                    if closedText ~= nil then
                        if i18n:hasText(closedText) then
                            closedText = i18n:getText(closedText)
                        end
                    end
                    self.openingHours = {startTime=startTime, endTime=endTime, disableIfClosed=disableIfClosed, closedText=closedText}
                    g_currentMission.environment:addHourChangeListener(self)
                end
				
                self.isEnabled = true

                local triggerId = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, key..".controls#triggerNode"))
				if triggerId ~= nil then
				
                    self.controls = {}
                    self.controls.triggerId = triggerId
					setCollisionMask(self.controls.triggerId, 74457088)
                    addTrigger(self.controls.triggerId, "triggerCallback", self)
					
                    for i=0, getNumOfChildren(self.controls.triggerId)-1 do
					    addTrigger(getChildAt(self.controls.triggerId, i), "triggerCallback", self)						
                    end

                    local posKey = getXMLString(xmlFile, key..".controls#posKey")
                    if posKey ~= nil then
                        if InputBinding[posKey] ~= nil then
                            self.controls.posKey = InputBinding[posKey]
                            local posText = getXMLString(xmlFile, key..".controls#posText")
                            if posText ~= nil then
                                if i18n:hasText(posText) then
                                    posText = i18n:getText(posText)
                                end
                                self.controls.posText = posText
                            end
                            local negText = getXMLString(xmlFile, key..".controls#negText")
                            if negText ~= nil then
                                if i18n:hasText(negText) then
                                    negText = i18n:getText(negText)
                                end
                                self.controls.negText = negText
                            end

                            local negKey = getXMLString(xmlFile, key..".controls#negKey")
                            if negKey ~= nil then
                                if InputBinding[negKey] ~= nil then
                                    self.controls.negKey = InputBinding[negKey]
                                else
                                    print("Warning: Negative direction key '"..negKey.."' not defined!")
                                end
                            end
                        else
                            print("Warning: Positive direction key '"..posKey.."' not defined!")
                        end
                    end
                end

                if g_client ~= nil then
                    self.sampleMoving = SoundUtil.loadSample(xmlFile, {}, key..".sound", nil, baseDir, self.nodeId)
                end
            else
                success = false
                print("Error: index '"..index.."' not found in AnimatedObject xml '"..self.xmlFilename.."'!")
            end
        else
            success = false
            print("Error: Missing 'index' user attribute for AnimatedObject node '"..getName(nodeId).."'!")
        end

        delete(xmlFile)
    end

    self.animatedObjectDirtyFlag = self:getNextDirtyFlag();

    return success
end

AnimatedObject.triggerCallback = function(self, triggerId, otherId, onEnter, onLeave, onStay)
	if g_currentMission.missionInfo:isa(FSCareerMissionInfo) then
	    local myVehicle = g_currentMission.nodeToVehicle[otherId];
		if onEnter or onLeave then
       	   	if g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
				if onEnter then
					self.playerInRange = true;
				elseif onLeave then
					self.playerInRange = nil;
				end;
			else
				if not g_currentMission.controlPlayer and myVehicle ~= nil then
				    if myVehicle:getIsActiveForInput(false) then
					    if onEnter then
							self.playerInRange = true;
				        elseif onLeave then
							self.playerInRange = nil;
						end;	
					end;
				end;
		   	end;
	   	end;
    end
end

end;
